"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmPlatform = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const ethers_1 = require("ethers");
const contracts_1 = require("./contracts");
const chain_1 = require("./chain");
const tokenBridge_1 = require("./protocols/tokenBridge");
const automaticTokenBridge_1 = require("./protocols/automaticTokenBridge");
const automaticCircleBridge_1 = require("./protocols/automaticCircleBridge");
const circleBridge_1 = require("./protocols/circleBridge");
const wormholeCore_1 = require("./protocols/wormholeCore");
/**
 * @category EVM
 */
class EvmPlatform {
    // Provides runtime concrete value
    static _platform = 'Evm';
    platform = EvmPlatform._platform;
    conf;
    contracts;
    constructor(conf) {
        this.conf = conf;
        this.contracts = new contracts_1.EvmContracts(conf);
    }
    getRpc(chain) {
        const rpcAddress = this.conf[chain].rpc;
        return ethers_1.ethers.getDefaultProvider(rpcAddress);
    }
    getChain(chain) {
        return new chain_1.EvmChain(this, chain);
    }
    getWormholeCore(rpc) {
        return wormholeCore_1.EvmWormholeCore.fromProvider(rpc, this.contracts);
    }
    async getTokenBridge(rpc) {
        return await tokenBridge_1.EvmTokenBridge.fromProvider(rpc, this.contracts);
    }
    async getAutomaticTokenBridge(rpc) {
        return await automaticTokenBridge_1.EvmAutomaticTokenBridge.fromProvider(rpc, this.contracts);
    }
    async getCircleBridge(rpc) {
        return await circleBridge_1.EvmCircleBridge.fromProvider(rpc, this.contracts);
    }
    async getAutomaticCircleBridge(rpc) {
        return await automaticCircleBridge_1.EvmAutomaticCircleBridge.fromProvider(rpc, this.contracts);
    }
    async getDecimals(chain, rpc, token) {
        if (token === 'native')
            return BigInt(this.conf[chain].nativeTokenDecimals);
        const tokenContract = this.contracts.mustGetTokenImplementation(rpc, token.address.toString());
        const decimals = await tokenContract.decimals();
        return decimals;
    }
    async getBalance(chain, rpc, walletAddr, tokenId) {
        if (tokenId === 'native')
            return await rpc.getBalance(walletAddr);
        const tb = await this.getTokenBridge(rpc);
        const address = await tb.getWrappedAsset(tokenId);
        if (!address)
            return null;
        const token = this.contracts.mustGetTokenImplementation(rpc, address.toString());
        const balance = await token.balanceOf(walletAddr);
        return balance;
    }
    async sendWait(rpc, stxns) {
        const txhashes = [];
        for (const stxn of stxns) {
            const txRes = await rpc.broadcastTransaction(stxn);
            const txReceipt = await txRes.wait();
            // TODO: throw error?
            if (txReceipt === null)
                continue;
            txhashes.push(txReceipt.hash);
        }
        return txhashes;
    }
    parseAddress(chain, address) {
        return (0, connect_sdk_1.toNative)(chain, address);
    }
    async parseTransaction(chain, rpc, txid) {
        const receipt = await rpc.getTransactionReceipt(txid);
        if (receipt === null)
            throw new Error(`No transaction found with txid: ${txid}`);
        const coreAddress = this.conf[chain].contracts.coreBridge;
        const coreImpl = this.contracts.getCoreImplementationInterface();
        return receipt.logs
            .filter((l) => {
            return l.address === coreAddress;
        })
            .map((log) => {
            const { topics, data } = log;
            const parsed = coreImpl.parseLog({ topics: topics.slice(), data });
            if (parsed === null)
                return undefined;
            const emitterAddress = this.parseAddress(chain, parsed.args.sender);
            return {
                chain: chain,
                emitter: emitterAddress.toUniversalAddress(),
                sequence: parsed.args.sequence,
            };
        })
            .filter(connect_sdk_1.isWormholeMessageId);
    }
}
exports.EvmPlatform = EvmPlatform;
//# sourceMappingURL=platform.js.map