"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmAutomaticTokenBridge = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const types_1 = require("../types");
const unsignedTransaction_1 = require("../unsignedTransaction");
class EvmAutomaticTokenBridge {
    network;
    chain;
    provider;
    contracts;
    tokenBridgeRelayer;
    chainId;
    // https://github.com/wormhole-foundation/wormhole-connect/blob/development/sdk/src/contexts/eth/context.ts#L379
    constructor(network, chain, provider, contracts) {
        this.network = network;
        this.chain = chain;
        this.provider = provider;
        this.contracts = contracts;
        this.chainId = (0, connect_sdk_1.evmNetworkChainToEvmChainId)(network, chain);
        this.tokenBridgeRelayer = this.contracts.mustGetTokenBridgeRelayer(chain, provider);
    }
    async *redeem(sender, vaa) {
        const senderAddr = (0, types_1.toEvmAddrString)(sender);
        const txReq = await this.tokenBridgeRelayer.completeTransferWithRelay.populateTransaction((0, connect_sdk_1.serialize)(vaa));
        return this.createUnsignedTx((0, types_1.addFrom)(txReq, senderAddr), 'TokenBridgeRelayer.completeTransferWithRelay');
    }
    static async fromProvider(provider, contracts) {
        const { chainId } = await provider.getNetwork();
        const networkChainPair = connect_sdk_1.evmChainIdToNetworkChainPair.get(chainId);
        if (networkChainPair === undefined)
            throw new Error(`Unknown EVM chainId ${chainId}`);
        const [network, chain] = networkChainPair;
        return new EvmAutomaticTokenBridge(network, chain, provider, contracts);
    }
    //alternative naming: initiateTransfer
    async *transfer(sender, recipient, token, amount, relayerFee, nativeGas) {
        const senderAddr = (0, types_1.toEvmAddrString)(sender);
        const recipientChainId = (0, connect_sdk_1.chainToChainId)(recipient.chain);
        const recipientAddress = recipient.address.toString();
        const nativeTokenGas = nativeGas ? nativeGas : 0n;
        if (token === 'native') {
            const txReq = await this.tokenBridgeRelayer.wrapAndTransferEthWithRelay.populateTransaction(nativeTokenGas, recipientChainId, recipientAddress, 0, // skip batching
            { value: relayerFee + amount + nativeTokenGas });
            yield this.createUnsignedTx((0, types_1.addFrom)(txReq, senderAddr), 'TokenBridgeRelayer.wrapAndTransferETHWithRelay');
        }
        else {
            //TODO check for ERC-2612 (permit) support on token?
            const tokenAddr = (0, types_1.toEvmAddrString)(token);
            // TODO: allowance?
            const txReq = await this.tokenBridgeRelayer.transferTokensWithRelay.populateTransaction(tokenAddr, amount, nativeTokenGas, recipientChainId, recipientAddress, 0);
            yield this.createUnsignedTx((0, types_1.addFrom)(txReq, senderAddr), 'TokenBridgeRelayer.transferTokensWithRelay');
        }
    }
    async getRelayerFee(sender, recipient, token) {
        const tokenId = token === 'native'
            ? await this.contracts.getNativeWrapped(sender.chain, this.provider)
            : token;
        const destChainId = (0, connect_sdk_1.toChainId)(recipient.chain);
        const destTokenAddress = (0, types_1.toEvmAddrString)(tokenId.address.toString());
        const tokenContract = this.contracts.mustGetTokenImplementation(this.provider, destTokenAddress);
        const decimals = await tokenContract.decimals();
        return await this.tokenBridgeRelayer.calculateRelayerFee(destChainId, destTokenAddress, decimals);
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new unsignedTransaction_1.EvmUnsignedTransaction((0, types_1.addChainId)(txReq, this.chainId), this.network, this.chain, description, parallelizable);
    }
}
exports.EvmAutomaticTokenBridge = EvmAutomaticTokenBridge;
//# sourceMappingURL=automaticTokenBridge.js.map