import { isWormholeMessageId, toNative, } from '@wormhole-foundation/connect-sdk';
import { ethers } from 'ethers';
import { EvmContracts } from './contracts';
import { EvmChain } from './chain';
import { EvmTokenBridge } from './protocols/tokenBridge';
import { EvmAutomaticTokenBridge } from './protocols/automaticTokenBridge';
import { EvmAutomaticCircleBridge } from './protocols/automaticCircleBridge';
import { EvmCircleBridge } from './protocols/circleBridge';
import { EvmWormholeCore } from './protocols/wormholeCore';
/**
 * @category EVM
 */
export class EvmPlatform {
    // Provides runtime concrete value
    static _platform = 'Evm';
    platform = EvmPlatform._platform;
    conf;
    contracts;
    constructor(conf) {
        this.conf = conf;
        this.contracts = new EvmContracts(conf);
    }
    getRpc(chain) {
        const rpcAddress = this.conf[chain].rpc;
        return ethers.getDefaultProvider(rpcAddress);
    }
    getChain(chain) {
        return new EvmChain(this, chain);
    }
    getWormholeCore(rpc) {
        return EvmWormholeCore.fromProvider(rpc, this.contracts);
    }
    async getTokenBridge(rpc) {
        return await EvmTokenBridge.fromProvider(rpc, this.contracts);
    }
    async getAutomaticTokenBridge(rpc) {
        return await EvmAutomaticTokenBridge.fromProvider(rpc, this.contracts);
    }
    async getCircleBridge(rpc) {
        return await EvmCircleBridge.fromProvider(rpc, this.contracts);
    }
    async getAutomaticCircleBridge(rpc) {
        return await EvmAutomaticCircleBridge.fromProvider(rpc, this.contracts);
    }
    async getDecimals(chain, rpc, token) {
        if (token === 'native')
            return BigInt(this.conf[chain].nativeTokenDecimals);
        const tokenContract = this.contracts.mustGetTokenImplementation(rpc, token.address.toString());
        const decimals = await tokenContract.decimals();
        return decimals;
    }
    async getBalance(chain, rpc, walletAddr, tokenId) {
        if (tokenId === 'native')
            return await rpc.getBalance(walletAddr);
        const tb = await this.getTokenBridge(rpc);
        const address = await tb.getWrappedAsset(tokenId);
        if (!address)
            return null;
        const token = this.contracts.mustGetTokenImplementation(rpc, address.toString());
        const balance = await token.balanceOf(walletAddr);
        return balance;
    }
    async sendWait(rpc, stxns) {
        const txhashes = [];
        for (const stxn of stxns) {
            const txRes = await rpc.broadcastTransaction(stxn);
            const txReceipt = await txRes.wait();
            // TODO: throw error?
            if (txReceipt === null)
                continue;
            txhashes.push(txReceipt.hash);
        }
        return txhashes;
    }
    parseAddress(chain, address) {
        return toNative(chain, address);
    }
    async parseTransaction(chain, rpc, txid) {
        const receipt = await rpc.getTransactionReceipt(txid);
        if (receipt === null)
            throw new Error(`No transaction found with txid: ${txid}`);
        const coreAddress = this.conf[chain].contracts.coreBridge;
        const coreImpl = this.contracts.getCoreImplementationInterface();
        return receipt.logs
            .filter((l) => {
            return l.address === coreAddress;
        })
            .map((log) => {
            const { topics, data } = log;
            const parsed = coreImpl.parseLog({ topics: topics.slice(), data });
            if (parsed === null)
                return undefined;
            const emitterAddress = this.parseAddress(chain, parsed.args.sender);
            return {
                chain: chain,
                emitter: emitterAddress.toUniversalAddress(),
                sequence: parsed.args.sequence,
            };
        })
            .filter(isWormholeMessageId);
    }
}
//# sourceMappingURL=platform.js.map