import { serialize, chainToChainId, evmChainIdToNetworkChainPair, evmNetworkChainToEvmChainId, toChainId, } from '@wormhole-foundation/connect-sdk';
import { addChainId, addFrom, toEvmAddrString, } from '../types';
import { EvmUnsignedTransaction } from '../unsignedTransaction';
export class EvmAutomaticTokenBridge {
    network;
    chain;
    provider;
    contracts;
    tokenBridgeRelayer;
    chainId;
    // https://github.com/wormhole-foundation/wormhole-connect/blob/development/sdk/src/contexts/eth/context.ts#L379
    constructor(network, chain, provider, contracts) {
        this.network = network;
        this.chain = chain;
        this.provider = provider;
        this.contracts = contracts;
        this.chainId = evmNetworkChainToEvmChainId(network, chain);
        this.tokenBridgeRelayer = this.contracts.mustGetTokenBridgeRelayer(chain, provider);
    }
    async *redeem(sender, vaa) {
        const senderAddr = toEvmAddrString(sender);
        const txReq = await this.tokenBridgeRelayer.completeTransferWithRelay.populateTransaction(serialize(vaa));
        return this.createUnsignedTx(addFrom(txReq, senderAddr), 'TokenBridgeRelayer.completeTransferWithRelay');
    }
    static async fromProvider(provider, contracts) {
        const { chainId } = await provider.getNetwork();
        const networkChainPair = evmChainIdToNetworkChainPair.get(chainId);
        if (networkChainPair === undefined)
            throw new Error(`Unknown EVM chainId ${chainId}`);
        const [network, chain] = networkChainPair;
        return new EvmAutomaticTokenBridge(network, chain, provider, contracts);
    }
    //alternative naming: initiateTransfer
    async *transfer(sender, recipient, token, amount, relayerFee, nativeGas) {
        const senderAddr = toEvmAddrString(sender);
        const recipientChainId = chainToChainId(recipient.chain);
        const recipientAddress = recipient.address.toString();
        const nativeTokenGas = nativeGas ? nativeGas : 0n;
        if (token === 'native') {
            const txReq = await this.tokenBridgeRelayer.wrapAndTransferEthWithRelay.populateTransaction(nativeTokenGas, recipientChainId, recipientAddress, 0, // skip batching
            { value: relayerFee + amount + nativeTokenGas });
            yield this.createUnsignedTx(addFrom(txReq, senderAddr), 'TokenBridgeRelayer.wrapAndTransferETHWithRelay');
        }
        else {
            //TODO check for ERC-2612 (permit) support on token?
            const tokenAddr = toEvmAddrString(token);
            // TODO: allowance?
            const txReq = await this.tokenBridgeRelayer.transferTokensWithRelay.populateTransaction(tokenAddr, amount, nativeTokenGas, recipientChainId, recipientAddress, 0);
            yield this.createUnsignedTx(addFrom(txReq, senderAddr), 'TokenBridgeRelayer.transferTokensWithRelay');
        }
    }
    async getRelayerFee(sender, recipient, token) {
        const tokenId = token === 'native'
            ? await this.contracts.getNativeWrapped(sender.chain, this.provider)
            : token;
        const destChainId = toChainId(recipient.chain);
        const destTokenAddress = toEvmAddrString(tokenId.address.toString());
        const tokenContract = this.contracts.mustGetTokenImplementation(this.provider, destTokenAddress);
        const decimals = await tokenContract.decimals();
        return await this.tokenBridgeRelayer.calculateRelayerFee(destChainId, destTokenAddress, decimals);
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new EvmUnsignedTransaction(addChainId(txReq, this.chainId), this.network, this.chain, description, parallelizable);
    }
}
//# sourceMappingURL=automaticTokenBridge.js.map